@@ -51,13 +51,13 @@ static CW_TABLE cw_table[] = {
 	/* Prosigns */
 	{'=',	"<BT>",   "-...-"	}, // 0
 	{'~',	"<AA>",   ".-.-"	}, // 1
-	{'%',	"<AS>",   ".-..."	}, // 2
-	{'+',	"<AR>",   ".-.-."	}, // 3
-	{'>',	"<SK>",   "...-.-"	}, // 4
-	{'<',	"<KN>",   "-.--."	}, // 5
+	{'<',	"<AS>",   ".-..."	}, // 2
+	{'>',	"<AR>",   ".-.-."	}, // 3
+	{'%',	"<SK>",   "...-.-"	}, // 4
+	{'+',	"<KN>",   "-.--."	}, // 5
 	{'&',	"<INT>",  "..-.-"	}, // 6
-	{'}',	"<HM>",   "....--"	}, // 7
-	{'{',	"<VE>",   "...-."	}, // 8
+	{'{',	"<HM>",   "....--"	}, // 7
+	{'}',	"<VE>",   "...-."	}, // 8
 	/* ASCII 7bit letters */
 	{'A',	"A",	".-"	},
 	{'B',	"B",	"-..."	},
