@@ -28,6 +28,7 @@
 // ----------------------------------------------------------------------------
 
 #include <config.h>
+#include <memory.h>
 
 #include <cmath>
 #include "misc.h"
@@ -52,10 +53,31 @@ fftfilt::fftfilt(double f1, double f2, int len)
 		ovlbuf[i].re = ovlbuf[i].im = 0.0;
 
 	inptr = 0;
-	
+
 	create_filter(f1, f2);
 }
 
+fftfilt::fftfilt(double f, int len)
+{
+	filterlen = len;
+	fft = new Cfft(filterlen);
+	ift = new Cfft(filterlen);
+
+	ovlbuf		= new complex[filterlen/2];
+	filter		= new complex[filterlen];
+	filtdata	= new complex[filterlen];
+	
+	for (int i = 0; i < filterlen; i++)
+		filter[i].re = filter[i].im =
+		filtdata[i].re = filtdata[i].im = 0.0;
+	for (int i = 0; i < filterlen/2; i++)
+		ovlbuf[i].re = ovlbuf[i].im = 0.0;
+
+	inptr = 0;
+
+	create_lpf(f);
+}
+
 fftfilt::~fftfilt()
 {
 	if (fft) delete fft;
@@ -75,7 +97,7 @@ void fftfilt::create_filter(double f1, double f2)
 	
 // initialize the filter to zero	
 	for (int i = 0; i < filterlen; i++)
-		filter[i].re = filter[i].im = 0.0;
+		filter[i].re   = filter[i].im   = 0.0;
 
 // create the filter shape coefficients by fft
 // filter values initialized to the impulse response h(t)
@@ -99,20 +121,52 @@ void fftfilt::create_filter(double f1, double f2)
 }
 
 
+void fftfilt::create_lpf(double f)
+{
+	int len = filterlen / 2 + 1;
+	double t, h, x, it;
+	Cfft *tmpfft;
+	tmpfft = new Cfft(filterlen);
+	
+// initialize the filter to zero	
+	for (int i = 0; i < filterlen; i++)
+		filter[i].re   = filter[i].im   = 0.0;
+
+// create the filter shape coefficients by fft
+// filter values initialized to the impulse response h(t)
+	for (int i = 0; i < len; i++) {
+		it = (double) i;
+		t = it - (len - 1) / 2.0;
+		h = it / (len - 1);
+
+		x = f * sinc(2 * f * t);
+		x *= blackman(h);	// windowed by Blackman function
+		x *= filterlen;		// scaled for unity in passband
+		filter[i].re = x;
+	}
+// perform the complex forward fft to obtain H(w)
+	tmpfft->cdft(filter);
+// start outputs after 2 full passes are complete
+	pass = 2;
+	delete tmpfft;
+}
+
+
 /*
  * Filter with fast convolution (overlap-add algorithm).
  */
 int fftfilt::run(const complex& in, complex **out)
 {
 // collect filterlen/2 input samples
+	const int filterlen_div2 = filterlen / 2 ;
 	filtdata[inptr++] = in;
 
-	if (inptr < filterlen / 2)
+	if (inptr < filterlen_div2)
 		return 0;
 	if (pass) --pass; // filter output is not stable until 2 passes
 
 // zero the rest of the input data
-	for (int i = filterlen / 2 ; i < filterlen; i++)
+	for (int i = filterlen_div2 ; i < filterlen; i++)
 		filtdata[i].re = filtdata[i].im = 0.0;
 	
 // FFT transpose to the frequency domain
@@ -127,15 +181,15 @@ int fftfilt::run(const complex& in, complex **out)
 	ift->icdft(filtdata);
 
 // overlap and add
-	for (int i = 0; i < filterlen / 2; i++) {
-		filtdata[i].re += ovlbuf[i].re;
-		filtdata[i].im += ovlbuf[i].im;
+	for (int i = 0; i < filterlen_div2; i++) {
+		filtdata[i] += ovlbuf[i];
 	}
 	*out = filtdata;
 
 // save the second half for overlapping
-	for (int i = 0; i < filterlen / 2; i++)
-		ovlbuf[i] = filtdata[i + filterlen / 2];
+	// Memcpy is allowed because complex are POD objects.
+	memcpy( ovlbuf, filtdata + filterlen_div2, sizeof( ovlbuf[0] ) * filterlen_div2 );
+
 
 // clear inbuf pointer
 	inptr = 0;
@@ -143,5 +197,5 @@ int fftfilt::run(const complex& in, complex **out)
 // signal the caller there is filterlen/2 samples ready
 	if (pass) return 0;
 	
-	return filterlen / 2;
+	return filterlen_div2;
 }
