@@ -113,7 +113,7 @@ public:
 	void		setFont(Fl_Font f, int attr = NATTR);
 
 protected:
-	enum { TX_MENU_TX, TX_MENU_RX, TX_MENU_ABORT, TX_MENU_MFSK16_WEFAX_IMG,
+	enum { TX_MENU_TX, TX_MENU_RX, TX_MENU_ABORT, TX_MENU_MFSK16_IMG,
 	       TX_MENU_CUT, TX_MENU_COPY, TX_MENU_PASTE, TX_MENU_CLEAR, TX_MENU_READ,
 	       TX_MENU_WRAP
 	};
