@@ -3,9 +3,11 @@
 
 class Socket;
 
-extern void WriteARQsocket(unsigned int data);
+extern void WriteARQsocket(unsigned char* data, size_t len);
 extern bool Socket_arqRx();
 extern void AbortARQ();
+extern void pskmail_notify_rsid(trx_mode mode);
+extern void pskmail_notify_s2n(double s2n_count, double s2n_avg, double s2n_stddev);
 
 class ARQ_SOCKET_Server
 {
