@@ -4,15 +4,7 @@
 #include <config.h>
 
 #if (FLDIGI_FLTK_API_MAJOR == 1 && FLDIGI_FLTK_API_MINOR < 3) || (FLARQ_FLTK_API_MAJOR == 1 && FLARQ_FLTK_API_MINOR < 3)
-//#ifdef __WIN32__
-//#  define FSEL_THREAD 1
-//#endif
-#define FSEL_THREAD 0
 class Fl_Native_File_Chooser;
-#else
-#include "FL/Fl_Native_File_Chooser.H"
-#define FSEL_THREAD 0
-#endif
 
 class FSEL
 {
@@ -42,4 +34,18 @@ private:
 	int result;
 };
 
+#else // API >=1.3.0
+
+namespace FSEL {
+
+	void create(void);
+	void destroy(void);
+	const char* select(const char* title, const char* filter, const char* def = 0, int *fsel = NULL);
+	const char* saveas(const char* title, const char* filter, const char* def = 0, int *fsel = NULL);
+	const char* dir_select(const char* title, const char* filter, const char* def = 0);
+
+}
+
+#endif // API < 1.3.0
+
 #endif // FILESELECT_H
