@@ -70,9 +70,21 @@ private:
 	}
 	inline double mac(const double *a, const double *b, unsigned int size) {
 		double sum = 0.0;
-		for (unsigned int i = 0; i < size; i++)
+		double sum2 = 0.0;
+		double sum3 = 0.0;
+		double sum4 = 0.0;
+		// Reduces read-after-write dependencies : Each subsum does not wait for the others.
+		// The CPU can therefore schedule each line independently.
+		for (; size > 3; size -= 4, a += 4, b+=4)
+		{
+			sum  += a[0] * b[0];
+			sum2 += a[1] * b[1];
+			sum3 += a[2] * b[2];
+			sum4 += a[3] * b[3];
+		}
+		for (; size; --size)
 			sum += (*a++) * (*b++);
-		return sum;
+		return sum + sum2 + sum3 + sum4 ;
 	}
 
 protected:
@@ -86,9 +98,9 @@ public:
 	void init_hilbert (int len, int dec);
 	double *bp_FIR(int len, int hilbert, double f1, double f2);
 	void dump();
-	int run (complex &in, complex &out);
-	int Irun (double &in, double &out);
-	int Qrun (double &in, double &out);
+	int run (const complex &in, complex &out);
+	int Irun (const double &in, double &out);
+	int Qrun (const double &in, double &out);
 };
 
 //=====================================================================
@@ -122,14 +134,14 @@ private:
 	int first;
 	int last;
 	int ptr;
-	complex *vrot;
-	complex *bins;
-	complex *delay;
+	struct vrot_bins_pair ;
+	vrot_bins_pair * __restrict__ vrot_bins ;
+	complex * __restrict__ delay;
 	double k2;
 public:
 	sfft(int len, int first, int last);
 	~sfft();
-	complex *run(const complex& input);
+	void run(const complex& input, complex * __restrict__ result, int stride );
 };
 
 
@@ -140,9 +152,6 @@ public:
 
 class goertzel {
 private:
-	double SR;
-	double FREQ;
-	double K;
 	int N;
 	int count;
 	double Q0;
