@@ -153,6 +153,11 @@ extern int Qidle_time;
 
 extern void toggleRSID();
 
+extern int rightof(Fl_Widget* w);
+extern int leftof(Fl_Widget* w);
+extern int above(Fl_Widget* w);
+extern int below(Fl_Widget* w);
+
 extern void set_menus();
 extern void create_fl_digi_main(int argc, char** argv);
 extern void update_main_title();
@@ -179,6 +184,12 @@ extern void set_rtty(double, double, double);
 extern void set_video(double *, int, bool = true);
 extern void set_zdata(complex *, int);
 
+extern void set_scope_xaxis_1(double y1);
+extern void set_scope_xaxis_2(double y2);
+extern void set_scope_yaxis_1(double x1);
+extern void set_scope_yaxis_2(double x2);
+extern void set_scope_clear_axis();
+
 extern void set_CWwpm();
 extern void put_rx_char(unsigned int data, int style = FTextBase::RECV);
 extern void put_sec_char( char chr );
@@ -253,6 +264,8 @@ extern void init_modem_squelch(trx_mode mode, int freq = 0);
 extern void start_tx();
 extern void abort_tx();
 
+extern void LOGGING_colors_font();
+
 extern void colorize_macro(int i);
 extern void colorize_macros();
 
@@ -306,6 +319,11 @@ extern void sync_cw_parameters();
 
 extern void open_recv_folder(const char *fname);
 
+// set notch parameter on flrig
+extern int  notch_frequency;
+extern void notch_on(int);
+extern void notch_off();
+
 // thread terminators
 extern void ADIF_RW_close(void);
 extern void EQSL_close(void);
@@ -314,4 +332,6 @@ extern void EQSL_close(void);
 extern Fl_Double_Window *notify_window;
 extern Fl_Double_Window *dxcc_window;
 
+extern void cb_mnuCheckUpdate(Fl_Widget *, void *);
+
 #endif
