@@ -24,6 +24,8 @@ extern cAdifIO       adifFile;
 extern std::string logbook_filename;
 extern std::string sDate_on;
 extern std::string sDate_off;
+extern std::string sTime_on;
+extern std::string sTime_off;
 
 extern void loadBrowser(bool keep_pos = false);
 
