@@ -1,7 +1,7 @@
+// ----------------------------------------------------------------------------
+// olivia.h
 //
-// Olivia.h
-//
-// Copyright (C) 2006
+// Copyright (C) 2006-2010
 //		Dave Freese, W1HKJ
 //
 // This file is part of fldigi.  Adapted from code contained in gmfsk source code 
@@ -9,27 +9,31 @@
 //	Copyright (C) 2005
 //	Tomi Manninen (oh2bns@sral.fi)
 //
-// fldigi is free software; you can redistribute it and/or modify
+// This file is part of fldigi.
+//
+// Fldigi is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
-// fldigi is distributed in the hope that it will be useful,
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with fldigi; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-//
-//
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
 
 #ifndef _OLIVIA_H
 #define _OLIVIA_H
 
 #include "modem.h"
 #include "jalocha/pj_mfsk.h"
+#include "sound.h"
+
+#define TONE_DURATION (SCBLOCKSIZE * 16)
+#define SR4 ((TONE_DURATION) / 4)
 
 class olivia : public modem {
 private:
@@ -45,22 +49,31 @@ private:
 	int			preamble;
 	unsigned int	shreg;
 
+	double		np;
+	double		sp;
+	double		sigpwr;
+	double		noisepwr;
+	
 	int			escape;
 	int			smargin;
 	int			sinteg;
 	int			tones;
 	int			bw;
+	double		tone_bw;
 
 	int			preamblesent;
 	int			postamblesent;
 	double		preamblephase;
 
 	double		txbasefreq;
+	double		tone_midfreq;
 	double		lastfreq;
-	
+
+	double		ampshape[SR4];
+	double		tonebuff[TONE_DURATION];
+
 	double		nco(double freq);
-	void		send_preamble();
-	void		send_postamble();
+	void		send_tones();
 	
 public:
 	olivia();
