@@ -10,36 +10,26 @@
 	#include "hamlib.h"
 #endif
 
-extern Fl_Double_Window *rigcontrol;
 extern std::string windowTitle;
 extern Cserial rigio;
 
 extern void initOptionMenus();
-extern void setMode();
-extern void setBW();
-extern void selBW(int);
-extern void selFreq(long int);
 extern void clearList();
 extern void updateSelect();
 extern size_t addtoList(long val);
 extern void buildlist();
-extern int  movFreq();
-extern int  qso_movFreq();
+extern void qso_movFreq(Fl_Widget* w, void*);
 extern int	cb_qso_opMODE();
 extern int  cb_qso_opBW();
 extern void qso_setMode();
 extern void setTitle();
 
-extern void selectFreq();
-extern void delFreq();
-extern void addFreq();
-extern void saveFreqList();
-
 extern void qso_addFreq();
 extern void qso_delFreq();
 extern void qso_selectFreq();
 extern void qso_setFreq();
 extern void qso_clearList();
+extern void saveFreqList();
 
 extern bool readRigXML();
 extern bool init_Xml_RigDialog();
@@ -52,6 +42,4 @@ extern void selMode(rmode_t m);
 extern std::string modeString(rmode_t m);
 #endif
 
-extern Fl_Double_Window * createRigDialog();
-
 #endif
