@@ -82,6 +82,7 @@ struct XMLRIG {
 	int     retries;
 	bool    echo;
 	bool    cmdptt;
+	bool    vsp;
 	void clear() {
 		port.clear();
 		baud = 1200;
@@ -93,6 +94,7 @@ struct XMLRIG {
 		rtscts = false;
 		echo = false;
 		cmdptt = false;
+		vsp = false;
 		write_delay = 0;
 		post_write_delay = 50;
 		timeout = 200;
