@@ -158,6 +158,7 @@ public:
 	void movetocenter();
 	void carrier(int cf);
 	int  carrier();
+	inline void  makeNotch_(int notch_frequency);
 	inline void makeMarker_(int width, const RGB* color, int freq, const RGB* clrMin, RGB* clrM, const RGB* clrMax);
 	void makeMarker();
 	void process_analog(double *sig, int len);
@@ -178,6 +179,8 @@ public:
 		drawMarker();};
 	int peakFreq(int f0, int delta);
 	double powerDensity(double f0, double bw);
+	double powerDensityMaximum(int bw_nb, const int (*bw)[2]) const ;
+
 	void setPrefilter(int v);
 	void setcolors();
 	double dFreq() {return dfreq;}
@@ -221,6 +224,7 @@ private:
 	RGB		*markerimage;
 	RGB		RGBmarker;
 	RGB		RGBcursor;
+	RGBI		RGBInotch;
 	double	*fftout;
     double  *fftwindow;
 	uchar	*scaleimage;
@@ -319,6 +323,11 @@ public:
 		return (wfdisp->powerDensity(f0,bw));
 	}
 
+	double powerDensityMaximum(int bw_nb, const int (*bw)[2]) const
+	{
+		return (wfdisp->powerDensityMaximum(bw_nb,bw));
+	}
+
 	int Speed();
 	void Speed(int rate);
 	int Mag();
