@@ -148,6 +148,7 @@ static void send_IPC_log(cQsoRec &rec)
 
 	int mm, dd, yyyy;
 	char szdate[9];
+	char sztime[5];
 	strncpy(szdate, rec.getField(QSO_DATE_OFF), 8);
 	szdate[8] = 0;
 	sscanf(&szdate[6], "%d", &dd); szdate[6] = 0;
@@ -159,8 +160,12 @@ static void send_IPC_log(cQsoRec &rec)
 	log_msg = log_msg + "program:"	+ PACKAGE_NAME + " v " 	+ PACKAGE_VERSION + LOG_MSEPARATOR;
 	addtomsg("version:",	LOG_MVERSION);
 	addtomsg("date:",		logdate.szDate(5));
-	addtomsg("time:", 		rec.getField(TIME_ON));
-	addtomsg("endtime:", 	rec.getField(TIME_OFF));
+	memset(sztime, 0, 5);
+	strncpy(sztime, rec.getField(TIME_ON), 4);
+	addtomsg("TIME:",		sztime);
+	memset(sztime, 0, 5);
+	strncpy(sztime, rec.getField(TIME_OFF), 4);
+	addtomsg("endtime:", 	sztime);
 	addtomsg("call:",		rec.getField(CALL));
 	addtomsg("mhz:",		rec.getField(FREQ));
 	addtomsg("mode:",		rec.getField(MODE));
