@@ -185,14 +185,32 @@ static void pWPM(std::string &s, size_t &i, size_t endbracket)
 		return;
 	}
 	int number;
-	std::string sTime = s.substr(i+5, endbracket - i - 5);
-	if (sTime.length() > 0) {
-		sscanf(sTime.c_str(), "%d", &number);
+	std::string snumber = s.substr(i+5, endbracket - i - 5);
+
+	if (snumber.length() > 0) {
+
+// first value = WPM
+		sscanf(snumber.c_str(), "%d", &number);
 		if (number < 5) number = 5;
 		if (number > 200) number = 200;
 		progdefaults.CWspeed = number;
 		sldrCWxmtWPM->value(number);
+printf("wpm %d\n", number);
+
+// second value = Farnsworth WPM
+		size_t pos;
+		if ((pos = snumber.find(":")) != std::string::npos) {
+			snumber.erase(0, pos+1);
+			if (snumber.length())
+				sscanf(snumber.c_str(), "%d", &number);
+			if (number < 15) number = 15;
+			if (number > 200) number = 200;
+			progdefaults.CWfarnsworth = number;
+			sldrCWfarnsworth->value(number);
+printf("fwpm %d\n", number);
+		}
 	}
+
 	s.replace(i, endbracket - i + 1, "");
 }
 
@@ -256,17 +274,40 @@ static void setwpm(int d)
 	cntCW_WPM->value(d);
 }
 
+static void setfwpm(int d)
+{
+	sldrCWfarnsworth->value(d);
+	progdefaults.CWusefarnsworth = true;
+	btnCWusefarnsworth->value(1);
+}
+
 static void doWPM(std::string s)
 {
 	int number;
-	std::string sTime = s.substr(6);
-	if (sTime.length() > 0) {
-		sscanf(sTime.c_str(), "%d", &number);
+	std::string snumber = s.substr(6);
+
+	if (snumber.length() > 0) {
+
+// first value = WPM
+		sscanf(snumber.c_str(), "%d", &number);
 		if (number < 5) number = 5;
 		if (number > 200) number = 200;
 		progdefaults.CWspeed = number;
 		REQ(setwpm, number);
+
+// second value = Farnsworth WPM
+		size_t pos;
+		if ((pos = snumber.find(":")) != std::string::npos) {
+			snumber.erase(0, pos+1);
+			if (snumber.length())
+				sscanf(snumber.c_str(), "%d", &number);
+			if (number < 15) number = 15;
+			if (number > 200) number = 200;
+			progdefaults.CWfarnsworth = number;
+			REQ(setfwpm, number);
+		}
 	}
+
 }
 
 static void pQueWPM(std::string &s, size_t &i, size_t endbracket)
@@ -561,22 +602,22 @@ static void pRST(std::string &s, size_t &i, size_t endbracket)
 
 static void pMYCALL(std::string &s, size_t &i, size_t endbracket)
 {
-	s.replace( i, 8, inpMyCallsign->value() );
+	s.replace( i, 8, progdefaults.myCall.c_str() );
 }
 
 static void pMYLOC(std::string &s, size_t &i, size_t endbracket)
 {
-	s.replace( i, 7, inpMyLocator->value() );
+	s.replace( i, 7, progdefaults.myLocator.c_str() );
 }
 
 static void pMYNAME(std::string &s, size_t &i, size_t endbracket)
 {
-	s.replace( i, 8, inpMyName->value() );
+	s.replace( i, 8, progdefaults.myName.c_str() );
 }
 
 static void pMYQTH(std::string &s, size_t &i, size_t endbracket)
 {
-	s.replace( i, 7, inpMyQth->value() );
+	s.replace( i, 7, progdefaults.myQth.c_str() );
 }
 
 static void pMYRST(std::string &s, size_t &i, size_t endbracket)
@@ -584,6 +625,11 @@ static void pMYRST(std::string &s, size_t &i, size_t endbracket)
 	s.replace( i, 7, inpRstIn->value() );
 }
 
+static void pANTENNA(std::string &s, size_t &i, size_t endbracket)
+{
+	s.replace( i, 9, progdefaults.myAntenna.c_str() );
+}
+
 static void pLDT(std::string &s, size_t &i, size_t endbracket)
 {
 	char szDt[80];
@@ -1619,19 +1665,19 @@ static void pEXEC(std::string &s, size_t &i, size_t endbracket)
 		return;
 	}
 
-	size_t start, end;
-	if ((start = s.find('>', i)) == std::string::npos ||
-		(end = s.rfind("</EXEC>")) == std::string::npos) {
+	size_t start = s.find(">", i);
+	size_t end = s.find("</EXEC>", start);
+
+	if (start == std::string::npos ||
+		end == std::string::npos) {
 		i++;
 		return;
 	}
-	start++;
-	i++;
 
-	std::string execstr = s.substr(start, end-start);
+	std::string execstr = s.substr(start+1, end-start-1);
 	within_exec = true;
 	MACROTEXT m;
-	execstr = m.expandMacro(execstr);
+	execstr = m.expandMacro(execstr, true);
 //	execstr.insert(0,ScriptsDir);
 	within_exec = false;
 
@@ -1657,8 +1703,12 @@ static void pEXEC(std::string &s, size_t &i, size_t endbracket)
 		perror("execl");
 		exit(EXIT_FAILURE);
 	}
-	// parent
+
+// parent
 	close(pfd[1]);
+	
+// give child process time to complete
+	MilliSleep(50);
 	FILE* fp = fdopen(pfd[0], "r");
 	if (!fp) {
 		LOG_PERROR("fdopen");
@@ -1666,26 +1716,26 @@ static void pEXEC(std::string &s, size_t &i, size_t endbracket)
 		return;
 	}
 
-	start = --i;
-	end = s.find('>', end) + 1;
-	s.erase(start, end-start);
+	s.erase(i, end - i + strlen("</EXEC>"));
+
 	char ln[BUFSIZ];
+	string lnbuff = "";
 	while (fgets(ln, sizeof(ln), fp)) {
-		end = strlen(ln);
-		s.insert(start, ln, end);
-		start += end;
+		lnbuff.append(ln);
 	}
-	// delete the trailing newline of what we read
-	if (start > i && s[start - 1] == '\n')
-		s.erase(start - 1, 1);
+// remove all trailing end-of-lines
+	while (lnbuff[lnbuff.length()-1] == '\n')
+		lnbuff.erase(lnbuff.length()-1,1);
+
+	if (!lnbuff.empty()) {
+		s.insert(i, lnbuff);
+		i += lnbuff.length();
+	} else
+		i++;
 
 	fclose(fp);
 	close(pfd[0]);
 
-	// what should we do with the shell-generated text?
-	// option 1: uncomment this line to skip & ignore it
-	// i = start;
-	// option 2: do nothing and allow it to be parsed for more macros
 }
 #else // !__MINGW32__
 
@@ -1706,7 +1756,7 @@ static void pEXEC(std::string& s, size_t& i, size_t endbracket)
 	std::string execstr = s.substr(start, end-start);
 	within_exec = true;
 	MACROTEXT m;
-	execstr = m.expandMacro(execstr);
+	execstr = m.expandMacro(execstr, true);
 	within_exec = false;
 
 	char* cmd = strdup(execstr.c_str());
@@ -1909,6 +1959,7 @@ static const MTAGS mtags[] = {
 {"<MYNAME>",	pMYNAME},
 {"<MYQTH>",		pMYQTH},
 {"<MYRST>",		pMYRST},
+{"<ANTENNA>",	pANTENNA},
 {"<QSOTIME>",	pQSOTIME},
 {"<INFO1>",		pINFO1},
 {"<INFO2>",		pINFO2},
@@ -2089,10 +2140,11 @@ void MACROTEXT::openMacroFile()
 {
 	std::string deffilename = MacrosDir;
 
-	if (progStatus.LastMacroFile.find("/") == string::npos)
-		deffilename.append(progStatus.LastMacroFile);
-	else
+	if (progStatus.LastMacroFile.find("/") != string::npos ||
+		progStatus.LastMacroFile.find("\\") != string::npos)
 		deffilename.assign(progStatus.LastMacroFile);
+	else
+		deffilename.append(progStatus.LastMacroFile);
 
 	const char *p = FSEL::select(
 			_("Open macro file"),
@@ -2113,10 +2165,12 @@ void MACROTEXT::openMacroFile()
 void MACROTEXT::writeMacroFile()
 {
 	std::string deffilename = MacrosDir;
-	if (progStatus.LastMacroFile.find("/") == string::npos)
-		deffilename.append(progStatus.LastMacroFile);
-	else
+	if (progStatus.LastMacroFile.find("/") != string::npos ||
+		progStatus.LastMacroFile.find("\\") != string::npos)
 		deffilename.assign(progStatus.LastMacroFile);
+	else
+		deffilename.append(progStatus.LastMacroFile);
+
 	saveMacros(deffilename.c_str());
 }
 
@@ -2124,10 +2178,11 @@ void MACROTEXT::saveMacroFile()
 {
 	std::string deffilename = MacrosDir;
 
-	if (progStatus.LastMacroFile.find("/") == string::npos)
-		deffilename.append(progStatus.LastMacroFile);
-	else
+	if (progStatus.LastMacroFile.find("/") != string::npos ||
+		progStatus.LastMacroFile.find("\\") != string::npos)
 		deffilename.assign(progStatus.LastMacroFile);
+	else
+		deffilename.append(progStatus.LastMacroFile);
 
 	const char *p = FSEL::saveas(
 			_("Save macro file"), 
@@ -2152,12 +2207,14 @@ void MACROTEXT::loadnewMACROS(std::string &s, size_t &i, size_t endbracket)
 	showMacroSet();
 }
 
-std::string MACROTEXT::expandMacro(std::string &s)
+std::string MACROTEXT::expandMacro(std::string &s, bool recurse = false)
 {
 	size_t idx = 0;
 	expand = true;
-	TransmitON = false;
-	ToggleTXRX = false;
+	if (!recurse) {
+		TransmitON = false;
+		ToggleTXRX = false;
+	}
 //	mNbr = n;
 	expanded = s;//text[n];
 	const MTAGS *pMtags;
