@@ -301,7 +301,7 @@ static void *rigMEM_loop(void *args)
 			break;
 
 		if (TogglePTT || rig_qsy || change_mode) {
-			IOout = fopen("c:/RIGCTL/ptt", "w");
+			IOout = fopen("c:/RIGCTL/ptt", "we");
 			if (IOout) {
 				LOG_VERBOSE("sent %d, %c, %s",
 					 (int)qsy_f,
@@ -318,7 +318,7 @@ static void *rigMEM_loop(void *args)
 			}
 		}
 
-		IOin = fopen("c:/RIGCTL/rig", "r");
+		IOin = fopen("c:/RIGCTL/rig", "re");
 		if (IOin) {
 			fscanf(IOin, "%ld\n", &IOfreq);
 			fscanf(IOin, "%s", szmode);
