@@ -58,6 +58,9 @@ modem *mfsk64_modem = 0;
 modem *wefax576_modem = 0;
 modem *wefax288_modem = 0;
 
+modem *navtex_modem = 0;
+modem *sitorb_modem = 0;
+
 modem *mt63_500_modem = 0;
 modem *mt63_1000_modem = 0;
 modem *mt63_2000_modem = 0;
