@@ -152,6 +152,10 @@ WFdisp::WFdisp (int x0, int y0, int w0, int h0, char *lbl) :
 	bandwidth = 32;
 	RGBmarker = RGBred;
 	RGBcursor = RGByellow;
+	RGBInotch.I = progdefaults.notchRGBI.I;
+	RGBInotch.R = progdefaults.notchRGBI.R;
+	RGBInotch.G = progdefaults.notchRGBI.G;
+	RGBInotch.B = progdefaults.notchRGBI.B;
 	mode = WATERFALL;
 	centercarrier = false;
 	overload = false;
@@ -439,6 +443,43 @@ double WFdisp::powerDensity(double f0, double bw)
 	return pwrdensity/(bw+1);
 }
 
+// Frequency of the maximum power for a given bandwidth. Used for AFC.
+double WFdisp::powerDensityMaximum(int bw_nb, const int (*bw)[2]) const
+{
+	double max_pwr = 0 ;
+	int f_lowest = bw[0][0];
+	int f_highest = bw[bw_nb-1][1];
+	if( f_lowest > f_highest ) abort();
+
+	for( int i = 0 ; i < bw_nb; ++i )
+	{
+		const int * p_bw = bw[i];
+		if( p_bw[0] > p_bw[1] ) abort();
+		for( int j = p_bw[0] ; j <= p_bw[1]; ++j )
+		{
+			max_pwr += pwr[ j - f_lowest ];
+		}
+	}
+
+	double curr_pwr = max_pwr ;
+	int max_idx = -1 ;
+	// Single pass to compute the maximum on this bandwidth.
+	for( int f = -f_lowest ; f < IMAGE_WIDTH - f_highest; ++f )
+	{
+		// Difference with previous power.
+		for( int i = 0 ; i < bw_nb; ++i )
+		{
+			const int * p_bw = bw[i];
+			curr_pwr += pwr[ f + p_bw[1] ] - pwr[ f + p_bw[0] ];
+		}
+		if( curr_pwr > max_pwr ) {
+			max_idx = f ;
+			max_pwr = curr_pwr ;
+		}
+	}
+	return max_idx ;
+}
+
 void WFdisp::setPrefilter(int v)
 {
 	switch (v) {
@@ -454,41 +495,44 @@ void WFdisp::setPrefilter(int v)
 int WFdisp::log2disp(int v)
 {
 	double val = 255.0 * (reflevel- v) / ampspan;
-	if (val < 0) val = 0;
-	if (val > 255 ) val = 255;
+	if (val < 0) return 255;
+	if (val > 255 ) return 0;
 	return (int)(255 - val);
 }
 
 void WFdisp::processFFT() {
-	int		n;
-	double scale;
 	int    ptrSample;
 	if (prefilter != progdefaults.wfPreFilter)
 	    setPrefilter(progdefaults.wfPreFilter);
 
-	scale = (double)SC_SMPLRATE / srate;
-	scale *= FFT_LEN / 2000.0;
+        const double scale = ( (double)SC_SMPLRATE / srate ) * ( FFT_LEN / 2000.0 );
 
 	if (dispcnt == 0) {
-		memset (fftout, 0, FFT_LEN*2*sizeof(double));
 		ptrSample = ptrCB;
 		int step = 8 / progdefaults.latency;
-		for (int i = 0; i < FFT_LEN * 2 / step; i++)
-			fftout[i] = fftwindow[i * step] * circbuff[i] * step;
+
+		int last_i = FFT_LEN * 2 / step;
+		for (int i = 0; i < last_i; i++)
+		fftout[i] = fftwindow[i * step] * circbuff[i] * step;
+		/// Zeroes only the last elements.
+		memset (fftout + last_i , 0, ( FFT_LEN*2 - last_i ) *sizeof(double));
+
 		wfft->rdft(fftout);
 FL_LOCK_D();
-		double pw;
-		int ffth;
-		for (int i = 0; i < IMAGE_WIDTH; i++) {
-			n = 2*(int)((scale * i / 2 + 0.5));
-			if (i <= progdefaults.LowFreqCutoff)
-				pw = 0.0;
-			else
-				pw = fftout[n]*fftout[n] + fftout[n+1]*fftout[n+1];
+		const int log2disp100 = log2disp(-100);
+		for (int i = 0; i <= progdefaults.LowFreqCutoff; i++) {
+			pwr[i] = 0.0;
+			fft_db[ptrFFTbuff * IMAGE_WIDTH + i] = log2disp100;
+		}
+
+		for (int i = progdefaults.LowFreqCutoff + 1; i < IMAGE_WIDTH; i++) {
+			int n = (int)(scale * i) & ~1 ; // Even number.
+			double pw = fftout[n]*fftout[n] + fftout[n+1]*fftout[n+1];
 			pwr[i] = pw;
-			ffth = (int)(10.0 * log10(pw + 1e-10) );
+			int ffth = (int)(10.0 * log10(pw + 1e-10) );
 			fft_db[ptrFFTbuff * IMAGE_WIDTH + i] = log2disp(ffth);
 		}
+
 		ptrFFTbuff--;
 		if (ptrFFTbuff < 0) ptrFFTbuff += image_height;
 FL_UNLOCK_D();
@@ -513,12 +557,12 @@ FL_UNLOCK_D();
 			dispcnt = wfspeed;
 		else if (srate == 11025)
 			dispcnt = wfspeed * 4 / 3;
+		//kl4yfd
 		else
-			dispcnt = wfspeed * 2;
+			dispcnt = wfspeed * 8 / 3;
 	}
 	--dispcnt;
 }
-
 void WFdisp::process_analog (double *sig, int len) {
 	int h1, h2, h3, sigw, sigy, sigpixel, ynext, graylevel;
 	h1 = h()/8 - 1;
@@ -744,8 +788,8 @@ void WFdisp::drawScale() {
 		    int cwoffset = 0;
 		    string testmode = qso_opMODE->value();
 		    if (testmode == "CW" or testmode == "CWR") {
-			cwoffset = progdefaults.CWsweetspot;
-			usb = ! (progdefaults.CWIsLSB ^ (testmode == "CWR")); 
+				cwoffset = ( progdefaults.CWOffset ? progdefaults.CWsweetspot : 0 );
+				usb = ! (progdefaults.CWIsLSB ^ (testmode == "CWR")); 
 		    }
 			if (usb)
 				fr = (rfc - (rfc%500))/1000.0 + 0.5*i - cwoffset/1000.0;
@@ -781,63 +825,44 @@ void WFdisp::drawMarker() {
 		step * RGBsize, RGBwidth);
 }
 
-
 void WFdisp::update_waterfall() {
 // transfer the fft history data into the WF image
-	short int *p1, *p2;
-	RGBI *p3, *p4;
+	short int * __restrict__ p1, * __restrict__ p2;
+	RGBI * __restrict__ p3, * __restrict__ p4;
 	p1 = tmp_fft_db + offset;
 	p2 = p1;
 	p3 = fft_img;
 	p4 = p3;
 
-	short* limit = tmp_fft_db + image_area - step + 1;
+	short*  __restrict__ limit = tmp_fft_db + image_area - step + 1;
 
-	for (int row = 0; row < image_height; row++) {
-		p2 = p1;
-		p4 = p3;
-		if (progdefaults.WFaveraging) {
-			if (step == 4)
-				for (int col = 0; col < disp_width; col++) {
-					*(p4++) = mag2RGBI[ (*p2+ *(p2+1)+ *(p2+2)+ *(p2+3))/4 ];
-					p2 += step;
-					if (p2 > limit) break;
-				}
-			else if (step == 2)
-				for (int col = 0; col < disp_width; col++) {
-					*(p4++) = mag2RGBI[ (*p2  + *(p2+1))/2 ];
-					p2 += step;
-					if (p2 > limit) break;
-				}
-			else
-				for (int col = 0; col < disp_width; col++) {
-					*(p4++) = mag2RGBI[ *p2 ];
-					p2 += step;
-					if (p2 > limit) break;
-				}
-		} else {
-			if (step == 4)
-				for (int col = 0; col < disp_width; col++) {
-					*(p4++) = mag2RGBI[ MAX( MAX ( MAX ( *p2, *(p2+1) ), *(p2+2) ), *(p2+3) ) ];
-					p2 += step;
-					if (p2 > limit) break;
-				}
-			else if (step == 2)
-				for (int col = 0; col < disp_width; col++) {
-					*(p4++) = mag2RGBI[ MAX( *p2, *(p2+1) ) ];
-					p2 += step;
-					if (p2 > limit) break;
-				}
-			else
-				for (int col = 0; col < disp_width; col++) {
-					*(p4++) = mag2RGBI[ *p2 ];
-					p2 += step;
-					if (p2 > limit) break;
-				}
+#define UPD_LOOP( Step, Operation ) \
+case Step: for (int row = 0; row < image_height; row++) { \
+		p2 = p1; \
+		p4 = p3; \
+		for ( const short *  __restrict__ last_p2 = std::min( p2 + Step * disp_width, limit +1 ); p2 < last_p2; p2 += Step ) { \
+			*(p4++) = mag2RGBI[ Operation ]; \
+		} \
+		p1 += IMAGE_WIDTH; \
+		p3 += disp_width; \
+	}; break
+
+	if (progdefaults.WFaveraging) {
+		switch(step) {
+			UPD_LOOP( 4, (*p2+ *(p2+1)+ *(p2+2)+ *(p2+3))/4 );
+			UPD_LOOP( 2, (*p2  + *(p2+1))/2 );
+			UPD_LOOP( 1, *p2 );
+			default:;
+		}
+	} else {
+		switch(step) {
+			UPD_LOOP( 4, MAX( MAX ( MAX ( *p2, *(p2+1) ), *(p2+2) ), *(p2+3) ) );
+			UPD_LOOP( 2, MAX( *p2, *(p2+1) ) );
+			UPD_LOOP( 1, *p2 );
+			default:;
 		}
-		p1 += IMAGE_WIDTH;
-		p3 += disp_width;
 	}
+#undef UPD_LOOP
 
 	if (progdefaults.UseBWTracks) {
 		int bw_lo = bandwidth / 2;
@@ -850,34 +875,51 @@ void WFdisp::update_waterfall() {
 		if (unlikely(pos2 == fft_img + disp_width))
 			pos2--;
 		if (likely(pos1 >= fft_img && pos2 < fft_img + disp_width)) {
-			for (int y = 0; y < image_height; y ++) {
-				if (mode == MODE_RTTY && progdefaults.useMARKfreq) {
-					if (active_modem->get_reverse()) {
-						*pos1 = progdefaults.rttymarkRGBI;
-						*pos2 = progdefaults.bwTrackRGBI;
-						if (progdefaults.UseWideTracks) {
-							*(pos1 + 1) = *pos1;
-							*(pos2 - 1) = *pos2;
-						}
-					} else {
-						*pos1 = progdefaults.bwTrackRGBI;
-						*pos2 = progdefaults.rttymarkRGBI;
-						if (progdefaults.UseWideTracks) {
-							*(pos1 + 1) = *pos1;
-							*(pos2 - 1) = *pos2;
-						}
-					}
+			RGBI rgbi1, rgbi2 ;
+
+			if (mode == MODE_RTTY && progdefaults.useMARKfreq) {
+				if (active_modem->get_reverse()) {
+					rgbi1 = progdefaults.rttymarkRGBI;
+					rgbi2 = progdefaults.bwTrackRGBI;
 				} else {
-					*pos1 = progdefaults.bwTrackRGBI;
-					*pos2 = progdefaults.bwTrackRGBI;
-					if (progdefaults.UseWideTracks) {
-						*(pos1 + 1) = *pos1;
-						*(pos2 - 1) = *pos2;
-					}
+					rgbi1 = progdefaults.bwTrackRGBI;
+					rgbi2 = progdefaults.rttymarkRGBI;
 				}
-				pos1 += disp_width;
-				pos2 += disp_width;
+			} else {
+				rgbi1 = progdefaults.bwTrackRGBI;
+				rgbi2 = progdefaults.bwTrackRGBI;
 			}
+			if (progdefaults.UseWideTracks) {
+				for (int y = 0; y < image_height; y ++) {
+					*(pos1 + 1) = *pos1 = rgbi1;
+					*(pos2 - 1) = *pos2 = rgbi2;
+					pos1 += disp_width;
+					pos2 += disp_width;
+				}
+			} else {
+				for (int y = 0; y < image_height; y ++) {
+					*pos1 = rgbi1;
+					*pos2 = rgbi2;
+					pos1 += disp_width;
+					pos2 += disp_width;
+				}
+			}
+		}
+	}
+
+// draw notch
+	if ((notch_frequency > 1) && (notch_frequency < progdefaults.HighFreqCutoff - 1)) {
+		RGBInotch.I = progdefaults.notchRGBI.I;
+		RGBInotch.R = progdefaults.notchRGBI.R;
+		RGBInotch.G = progdefaults.notchRGBI.G;
+		RGBInotch.B = progdefaults.notchRGBI.B;
+		RGBI  *notch = fft_img + (notch_frequency - offset) / step;
+		int dash = 0;
+		for (int y = 0; y < image_height; y++) {
+			dash = (dash + 1) % 6;
+			if (dash == 0 || dash == 1 || dash == 2)
+				*(notch-1) = *notch = *(notch+1) = RGBInotch;
+			notch += disp_width;
 		}
 	}
 }
@@ -1029,6 +1071,18 @@ void WFdisp::drawspectrum() {
 		}
 	}
 
+// draw notch
+	if ((notch_frequency > 1) && (notch_frequency < progdefaults.HighFreqCutoff - 1)) {
+		uchar  *notch = pixmap + (notch_frequency - offset) / step;
+		int dash = 0;
+		for (int y = 0; y < image_height; y++) {
+			dash = (dash + 1) % 6;
+			if (dash == 0 || dash == 1 || dash == 2)
+				*(notch-1) = *notch = *(notch+1) = 255;
+			notch += IMAGE_WIDTH/step;
+		}
+	}
+
 	fl_color(FL_BLACK);
 	fl_rectf(x(), y(), w(), WFSCALE + WFMARKER + WFTEXT + image_height);
 
@@ -1054,7 +1108,7 @@ void WFdisp::draw() {
 	checkWidth();
 	switch (mode) {
 	case SPECTRUM :
-			drawspectrum();
+		drawspectrum();
 		drawMarker();
 		break;
 	case SCOPE :
@@ -1062,10 +1116,10 @@ void WFdisp::draw() {
 		break;
 	case WATERFALL :
 	default:
-			if (dispcolor)
-				drawcolorWF();
-			else
-				drawgrayWF();
+		if (dispcolor)
+			drawcolorWF();
+		else
+			drawgrayWF();
 		drawMarker();
 	}
 }
@@ -1859,6 +1913,13 @@ int WFdisp::handle(int event)
 			}
 			goto lrclick;
 		case FL_LEFT_MOUSE:
+			if ((Fl::event_state() & (FL_ALT | FL_CTRL)) == (FL_ALT | FL_CTRL)) {
+				if (notch_frequency)
+					notch_off();
+				else
+					notch_on(cursorFreq(xpos));
+				return 1;
+			}
 			if (event == FL_PUSH) {
 				push = ypos;
 				pxpos = xpos;
